# Differential Testing for eBPF

## Setting up your environment

Install the dependencies listed in `requirements.txt`.
```bash
pip3 install -r requirements.txt
```

Set the environment variable with the OpenAI API key. 
```bash
export OPENAI_API_KEY="<ENTER OPENAI API KEY>"
```


## 1. Extract instructions
Run `python main_map_instructions_list_to_testfile_name.py` from the `\nl_to_test_map\` directory to generate a mapping between instructions and test files from the bpf_conformance repo.

## 2. Bug categories
Run `python main_high_level_categories.py` from the `\bug_categories\` directory to get the high level categories of bug classes from historical bugs dataset.
 
## 3. Test Generation
The test generation scripts includes all the components in the prompting framework, including extraction of constraints from ISA, extracting relevant code, generating test descriptions, and generating the actual tests themselves. Some scripts have few more components based on the abalation being run. See Section 4.3 in paper for description of all ablations.

```python
# list of ablations include the following
ablations = ['3shot_random', 'target_section', 'random_instruction_permutation_3', 'random_instruction_permutation_5', 'random_instruction_permutation_10',  'prompt_chain', 'prompt_chain_instruct', 'bug_centric', 'code_description', 'code_description_diff', 'code_diff', 'bug_guided_code_diff']
```

All the different abalations are under the `\test_generation` directory. Create the output directory for the generated tests under `\test_generation\output\<abalation>\processed\`. To run them, just use 
```bash
python -u main_<ablation>.py 2>&1 | tee output_<abalation>.txt
```

## 4. Utils
The data loaders and other data specific utilities are in `utils\data_utils.py`

The model loaders and other model specific utilities are in `utils\model_utils.py`

All the prompts for all abalations are in `utils\prompt_utils.py`

## 5. Evaluation

### 5.1 Setup instructions for eBPF
We setup the different eBPF implementations, as well as the plugin for bpf_conformance to run the tests on the different implementations by following the guidelines in the respective repositories. Note that the bpf conformance
- [uBPF](https://github.com/iovisor/ubpf)
- [ebpf for windows](https://github.com/microsoft/ebpf-for-windows)
- [bpf conformance](https://github.com/Alan-Jowett/bpf_conformance)

### 5.2 Run tests

For linux, copy file `run_tests_for_linux.py` to bpf_conformance folder. Then use `python3 run_tests_for_linux.py` to execute tests on linux.

For ubpf, copy file `run_tests_with_ubpf.py` to the parent of bpf_conformance folder. Then use `python3 run_tests_with_ubpf.py` to execute tests on ubpf. This assumes that the bpf_conformance directory and the ubpf directory are in the same directory level.

For windows, copy file `run_tests_for_windows.py` to `ebpf-for-windows\x64\Debug` folder. Then use `python3 run_tests_for_windows.py` to execute tests on windows.



## 6. Analysis
First run `python analyse_logs.py` to get all the summary results and numbers to analyse the outputs for a single implementation.

Run `python differential_testing.py` to get all the numbers for differential behaviour across two implementations. (need to specify which 2 implementations to run it on under main function)

Run `python compare_all_implementations.py` to get diff table for all three implementations.

Run `python analyse_generated_tests.py` to get the distribution of number of lines in generated tests.


## Misc

`\code\context\` has all the relevant context for prompts including the ISA doc, and instructions list.  
`\code\examples\` has all the random few shot examples used in prompts.


## References
- [ebpf](https://ebpf.io/)
- [bpf conformance](https://github.com/Alan-Jowett/bpf_conformance)
- [uBPF](https://github.com/iovisor/ubpf)
- [ebpf for windows](https://github.com/microsoft/ebpf-for-windows)
