import pandas as pd
import sys
sys.path.append('../')
from utils.data_utils import load_file

def load_data(path):
    df = pd.read_json(path)
    return df


#given the test set and the logs for linux and windows on the same test set can we find discrepancies in outcome


def main(baseline, path, results):
    
    
    print(f'\n\n\nbaseline type: {baseline}')

    df1 = load_data(f'{path}/processed_output_{baseline}_windows.json')
    df2 = load_data(f'{path}/processed_output_{baseline}_linux.json')
    df3 = load_data(f'{path}/processed_output_{baseline}_ubpf.json')

    df3.columns = [col_name+'_ubpf' for col_name in df3.columns]
    joint_df = df1.join(df2, lsuffix='_windows', rsuffix='_linux')
    joint_df = joint_df.join(df3)

    joint_df['outputs_match'] = joint_df.apply(lambda x: (x.test_status_windows == x.test_status_linux == x.test_status_ubpf) & (x.test_status_windows == x.test_status_linux == x.test_status_ubpf), axis=1)
    
    print(joint_df.columns)
    joint_df['file_content'] = joint_df.apply(lambda x: load_file(f"../../data/all_generated_tests/{baseline}/processed/{x.test_filename_windows}"), axis=1)
    
    matching_joint_df = joint_df[joint_df['outputs_match'] == True]
    mismatch_joint_df = joint_df[joint_df['outputs_match'] == False]
    print(f'joint_df.columns: {joint_df.columns}')
    print(f'len(joint_df): {len(joint_df)}')
    print(f'Test status match for all implementations: {len(matching_joint_df)}')
    print(f'Test status mismatch for implementations: {len(mismatch_joint_df)}')

    print(f"All PASS: {len(matching_joint_df[matching_joint_df['test_status_windows'] == 'PASS'])}")
    print(f"All FAIL: {len(matching_joint_df[matching_joint_df['test_status_windows'] == 'FAIL'])}")
    print(f"All SKIP: {len(matching_joint_df[matching_joint_df['test_status_windows'] == 'SKIP'])}")
    print(f"All ERROR: {len(matching_joint_df[matching_joint_df['test_status_windows'] == 'ERROR'])}")
    print(f"All CRASH: {len(matching_joint_df[matching_joint_df['test_status_windows'] == 'CRASH'])}")

    all_fail = matching_joint_df[matching_joint_df['test_status_windows'] == 'FAIL']
    
    all_fail['return_val_match'] = joint_df.apply(lambda x: x.return_value_windows == x.return_value_linux == x.return_value_ubpf, axis=1)

    all_fail['return_matches_expected'] = joint_df.apply(lambda x: x.return_value_windows == x.return_value_linux == x.return_value_ubpf == x.expected_value_windows, axis=1)

    all_fail['return_mismatches_expected'] = joint_df.apply(lambda x: x.return_value_windows == x.return_value_linux == x.return_value_ubpf != x.expected_value_windows, axis=1)

    index = len(results)
    total = len(joint_df)
    #update results_df
    results.loc[index, 'baseline'] = baseline
    results.loc[index, 'total'] = len(joint_df)
    results.loc[index, 'all_match_status'] = len(matching_joint_df)
    results.loc[index, 'all_match_status_percentage'] = len(matching_joint_df)/total*100
    results.loc[index, 'mismatch_status'] = len(mismatch_joint_df)
    results.loc[index, 'mismatch_status_percentage'] = len(mismatch_joint_df)/total*100
    results.loc[index, 'all_mismatches'] = len(mismatch_joint_df) + len(all_fail[all_fail['return_val_match'] == False])
    results.loc[index, 'all_mismatches_percentage'] = (len(mismatch_joint_df) + len(all_fail[all_fail['return_val_match'] == False]))/total*100
    results.loc[index, 'all_pass'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'PASS'])
    results.loc[index, 'all_pass_percentage'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'PASS'])/total*100
    results.loc[index, 'all_fail'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'FAIL'])
    results.loc[index, 'all_fail_percentage'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'FAIL'])/total*100
    results.loc[index, 'all_skip'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'SKIP'])
    results.loc[index, 'all_skip_percentage'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'SKIP'])/total*100
    results.loc[index, 'all_error'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'ERROR'])
    results.loc[index, 'all_error_percentage'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'ERROR'])/total*100
    results.loc[index, 'all_crash'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'CRASH'])
    results.loc[index, 'all_crash_percentage'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'CRASH'])/total*100
    results.loc[index, 'fail_return_val_mismatch'] = len(all_fail[all_fail['return_val_match'] == False])
    results.loc[index, 'fail_return_val_match'] = len(all_fail[all_fail['return_val_match'] == True])
    results.loc[index, 'fail_return_matches_expected'] = len(all_fail[all_fail['return_matches_expected'] == True])
    results.loc[index, 'fail_return_mismatches_expected'] = len(all_fail[all_fail['return_mismatches_expected'] == True])

    results.loc[index, 'valid (all_pass + all_fail_return_value_match)'] = len(matching_joint_df[matching_joint_df['test_status_windows'] == 'PASS']) + len(all_fail[all_fail['return_val_match'] == True])

    results.loc[index, 'valid_percentage'] = (len(matching_joint_df[matching_joint_df['test_status_windows'] == 'PASS']) + len(all_fail[all_fail['return_val_match'] == True]))/len(joint_df)* 100

    mismatch_joint_df = pd.concat([mismatch_joint_df, all_fail[all_fail['return_val_match'] == False]])
    
    mismatch_joint_df['baseline'] = baseline
    matching_joint_df['baseline'] = baseline


    return results, mismatch_joint_df, matching_joint_df


def fill_in_missing_columns_in_diff(implementation, diff_tests, mismatch_tests):
    for i in range(len(diff_tests)):
        row = diff_tests.loc[i]
        print(f"row['test_status_{implementation}']", row[f'test_status_{implementation}'])
        if pd.isna(row[f'test_status_{implementation}']):
            print(row)
            print(row['baseline'])
            print(mismatch_tests['baseline'])
            
            print(mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename'])])

            print(mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'test_status_{implementation}'])

            diff_tests.loc[i, f'test_status_{implementation}'] = mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'test_status_{implementation}'].values[0]
     
            print(mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'test_status_{implementation}'].values)
            

        print(f"row['test_outcome_{implementation}']", row[f'test_outcome_{implementation}'])
        if pd.isna(row[f'test_outcome_{implementation}']):
            diff_tests.loc[i, f'test_outcome_{implementation}'] = mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'test_outcome_{implementation}'].values[0]
            print(mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'test_outcome_{implementation}'].values[0])
            

        print(f"row['return_val_{implementation}']", row[f'return_val_{implementation}'])
        if pd.isna(row[f'return_val_{implementation}']):
            diff_tests.loc[i, f'return_val_{implementation}'] = mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'return_value_{implementation}'].values[0]
            print(mismatch_tests[(mismatch_tests[f'test_filename_{implementation}'] == row['filename']) & (mismatch_tests['baseline'] == row['baseline'])][f'return_value_{implementation}'].values[0])
        
        print('\n\n')
    
    return diff_tests  

if __name__ == "__main__":
    path = './output_logs'
    
    baselines = ['3shot_random', 'target_section', 'random_instruction_permutation_3', 'random_instruction_permutation_5', 'random_instruction_permutation_10',  'prompt_chain', 'prompt_chain_instruct', 'bug_centric', 'code_description', 'code_description_diff', 'code_diff', 'bug_guided_code_diff']

    res_columns = ['baseline', 'total', 'all_match_status', 'all_match_status_percentage', 'mismatch_status', 'mismatch_status_percentage', 'all_mismatches', 'all_mismatches_percentage', 'all_pass', 'all_pass_percentage', 'all_fail', 'all_fail_percentage', 'all_skip', 'all_skip_percentage', 'all_error', 'all_error_percentage', 'all_crash', 'all_crash_percentage', 'valid (all_pass + all_fail_return_value_match)', 'valid_percentage', 'fail_return_val_mismatch', 'fail_return_val_match', 'fail_return_matches_expected','fail_return_mismatches_expected']

    results = pd.DataFrame(columns = res_columns)
    
    mismatch_tests = pd.DataFrame(columns=['baseline','index_windows', 'test_file_path_windows', 'test_filename_windows', 'output_windows', 'error_message_windows', 'instruction_windows','test_status_windows', 'test_outcome_windows', 'test_outcome_class_windows', 'expected_value_windows', 'return_value_windows', 'index_linux', 'test_file_path_linux', 'test_filename_linux', 'output_linux', 'error_message_linux', 'instruction_linux', 'test_status_linux', 'test_outcome_linux', 'test_outcome_class_linux', 'expected_value_linux','return_value_linux', 'index_ubpf', 'test_file_path_ubpf', 'test_filename_ubpf', 'output_ubpf', 'error_message_ubpf', 'instruction_ubpf', 'test_status_ubpf', 'test_outcome_ubpf', 'test_outcome_class_ubpf', 'expected_value_ubpf', 'return_value_ubpf','outputs_match'])
    
    match_tests = pd.DataFrame(columns=['baseline','index_windows', 'test_file_path_windows', 'test_filename_windows', 'output_windows', 'error_message_windows', 'instruction_windows','test_status_windows', 'test_outcome_windows', 'test_outcome_class_windows', 'expected_value_windows', 'return_value_windows', 'index_linux', 'test_file_path_linux', 'test_filename_linux', 'output_linux', 'error_message_linux', 'instruction_linux', 'test_status_linux', 'test_outcome_linux', 'test_outcome_class_linux', 'expected_value_linux','return_value_linux', 'index_ubpf', 'test_file_path_ubpf', 'test_filename_ubpf', 'output_ubpf', 'error_message_ubpf', 'instruction_ubpf', 'test_status_ubpf', 'test_outcome_ubpf', 'test_outcome_class_ubpf', 'expected_value_ubpf', 'return_value_ubpf','outputs_match'])

    for baseline in baselines:
        results, mismatch_joint_df, matching_joint_df = main(baseline, path, results)
        mismatch_tests = pd.concat([mismatch_tests, mismatch_joint_df])
        match_tests = pd.concat([match_tests, matching_joint_df])
    
    print(results)
    results.to_csv(f'{path}/test_status_summary.csv')
    
    mismatch_tests.to_csv(f'{path}/mismatch_tests.csv')
    print(f'total mismatches: {len(mismatch_tests)}')    

    match_tests.to_csv(f'{path}/match_tests.csv')
    print(f'total matches: {len(match_tests)}')    
    
    
    # diffs by default only compares 2 implementations, add the third when missing
    diff_tests = pd.read_csv(f'diff_tests_manual_labels.csv')
    mismatch_tests = pd.read_csv(f'{path}/mismatch_tests.csv')

    for implementation in ['linux', 'windows', 'ubpf']:
        diff_tests = fill_in_missing_columns_in_diff(implementation, diff_tests, mismatch_tests)
    diff_tests.to_csv(f'./diff_tests_all_values.csv')
    
    
# to run 
# python compare_all_implementations.py | tee output_compare_all_implementations.txt

