import pandas as pd
import sys
sys.path.append('../')
from utils.data_utils import load_file, save_file

def load_data(path):
    df = pd.read_json(path)
    return df


#given the test set and the logs for linux and windows on the same test set can we find discrepancies in outcome


def main(baseline, implementation_type_1, implementation_type_2, results_df, path):

  
    print(f'\n\n\nbaseline type: {baseline}')

    df1 = load_data(f'{path}/processed_output_{baseline}_{implementation_type_1}.json')
    df2 = load_data(f'{path}/processed_output_{baseline}_{implementation_type_2}.json')

    joint_df = df1.join(df2, lsuffix=f'_{implementation_type_1}', rsuffix=f'_{implementation_type_2}')

    print(joint_df)

    diff_df = joint_df[joint_df[f'test_status_{implementation_type_1}'] != joint_df[f'test_status_{implementation_type_2}']]

    print(f'diff in {implementation_type_2} vs {implementation_type_1}', len(diff_df))

    diff_df3 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] != 'SKIP')] 
    diff_df3 = diff_df3[(diff_df3[f'test_status_{implementation_type_2}'] != 'SKIP')]
    
    diff_df8 = diff_df[(diff_df[f'test_status_{implementation_type_2}'] == 'SKIP')]
    print(f'{implementation_type_2} skip', len(diff_df8))
    
    diff_df9 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'SKIP')]
    print(f'{implementation_type_1} skip', len(diff_df9))
    
    print(f'diff with no skips', len(diff_df3))

    # row 1: Pass/Pass, Pass/Fail, Pass/Skip, Pass/Error
    diff_df11 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'PASS') & (joint_df[f'test_status_{implementation_type_2}'] == 'PASS')]
    print(f'{implementation_type_1} & {implementation_type_2} pass', len(diff_df11))

    diff_df1 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'PASS') & (diff_df[f'test_status_{implementation_type_2}'] == 'FAIL')]
    print(f'{implementation_type_1} pass & {implementation_type_2} fail', len(diff_df1))
    print(diff_df1)

    diff_df16 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'PASS') & (diff_df[f'test_status_{implementation_type_2}'] == 'SKIP')]
    print(f'{implementation_type_1} pass & {implementation_type_2} skip', len(diff_df16))
    
    diff_df6 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'PASS') & (diff_df[f'test_status_{implementation_type_2}'] == 'ERROR')]
    print(f'{implementation_type_1} pass & {implementation_type_2} error', len(diff_df6))
    
    diff_df28 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'PASS') & (diff_df[f'test_status_{implementation_type_2}'] == 'CRASH')]
    print(f'{implementation_type_1} pass & {implementation_type_2} crash', len(diff_df28))
    
    

    # row 2: Fail/Pass, Fail/Fail, Fail/Skip, Fail/Error
    diff_df2 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'FAIL') & (diff_df[f'test_status_{implementation_type_2}'] == 'PASS')]
    print(f'{implementation_type_1} fail & {implementation_type_2} pass', len(diff_df2))
    print(diff_df2)
    
    diff_df12 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'FAIL') & (joint_df[f'test_status_{implementation_type_2}'] == 'FAIL')]
    print(f'{implementation_type_1} & {implementation_type_2} fail', len(diff_df12))

    diff_df17 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'FAIL') & (diff_df[f'test_status_{implementation_type_2}'] == 'SKIP')]
    print(f'{implementation_type_1} fail & {implementation_type_2} skip', len(diff_df17))
    
    diff_df7 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'FAIL') & (diff_df[f'test_status_{implementation_type_2}'] == 'ERROR')]
    print(f'{implementation_type_1} fail & {implementation_type_2} error', len(diff_df7))
    
    diff_df27 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'FAIL') & (diff_df[f'test_status_{implementation_type_2}'] == 'CRASH')]
    print(f'{implementation_type_1} fail & {implementation_type_2} crash', len(diff_df27))
    

    # row 3: Skip/Pass, Skip/Fail, Skip/Skip, Skip/Error
    diff_df14 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'SKIP') & (diff_df[f'test_status_{implementation_type_2}'] == 'PASS')]
    print(f'{implementation_type_1} skip & {implementation_type_2} pass', len(diff_df14))
    
    diff_df15 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'SKIP') & (diff_df[f'test_status_{implementation_type_2}'] == 'FAIL')]
    print(f'{implementation_type_1} skip & {implementation_type_2} fail', len(diff_df15))

    diff_df10 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'SKIP') & (joint_df[f'test_status_{implementation_type_2}'] == 'SKIP')]
    print(f'{implementation_type_1} & {implementation_type_2} Skip', len(diff_df10))

    diff_df19 = diff_df[(diff_df[f'test_status_{implementation_type_2}'] == 'SKIP') & (diff_df[f'test_status_{implementation_type_2}'] == 'ERROR')]
    print(f'{implementation_type_1} skip & {implementation_type_2} error', len(diff_df19))
    
    diff_df26 = diff_df[(diff_df[f'test_status_{implementation_type_2}'] == 'SKIP') & (diff_df[f'test_status_{implementation_type_2}'] == 'CRASH')]
    print(f'{implementation_type_1} skip & {implementation_type_2} crash', len(diff_df26))
    

    # row 4: Error/Pass, Error/Fail, Error/Skip, Error/Error
    diff_df4 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'ERROR') & (diff_df[f'test_status_{implementation_type_2}'] == 'PASS')]
    print(f'{implementation_type_1} error & {implementation_type_2} pass', len(diff_df4))
    
    diff_df5 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'ERROR') & (diff_df[f'test_status_{implementation_type_2}'] == 'FAIL')]
    print(f'{implementation_type_1} error & {implementation_type_2} fail', len(diff_df5))
    
    diff_df18 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'ERROR') & (diff_df[f'test_status_{implementation_type_2}'] == 'SKIP')]
    print(f'{implementation_type_1} error & {implementation_type_2} skip', len(diff_df18))

    diff_df13 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'ERROR') & (joint_df[f'test_status_{implementation_type_2}'] == 'ERROR')]
    print(f'{implementation_type_1} & {implementation_type_2} error', len(diff_df13))

    diff_df25 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'ERROR') & (joint_df[f'test_status_{implementation_type_2}'] == 'CRASH')]
    print(f'{implementation_type_1} error & {implementation_type_2} crash', len(diff_df25))


    # row 4: Crash/Pass, Crash/Fail, Crash/Skip, Crash/Error, Crash/Crash
    diff_df20 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'CRASH') & (diff_df[f'test_status_{implementation_type_2}'] == 'PASS')]
    print(f'{implementation_type_1} crash & {implementation_type_2} pass', len(diff_df20))
    
    diff_df21 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'CRASH') & (diff_df[f'test_status_{implementation_type_2}'] == 'FAIL')]
    print(f'{implementation_type_1} crash & {implementation_type_2} fail', len(diff_df21))
    
    diff_df22 = diff_df[(diff_df[f'test_status_{implementation_type_1}'] == 'CRASH') & (diff_df[f'test_status_{implementation_type_2}'] == 'SKIP')]
    print(f'{implementation_type_1} crash & {implementation_type_2} skip', len(diff_df22))

    diff_df23 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'CRASH') & (joint_df[f'test_status_{implementation_type_2}'] == 'ERROR')]
    print(f'{implementation_type_1} crash & {implementation_type_2} error', len(diff_df23))

    diff_df24 = joint_df[(joint_df[f'test_status_{implementation_type_1}'] == 'CRASH') & (joint_df[f'test_status_{implementation_type_2}'] == 'CRASH')]
    print(f'{implementation_type_1} & {implementation_type_2} crash', len(diff_df24))


    # mismatched failures    
    diff_fail = diff_df12
    
    diff_fail = diff_fail[diff_fail[f'return_value_{implementation_type_1}'] !=  diff_fail[f'return_value_{implementation_type_2}']]
    print(f'{implementation_type_1} & {implementation_type_2} num mismatched failures', len(diff_fail))
    
    
    for i, row in diff_fail.iterrows():
        print(f"{i}: test_filename: {row[f'test_filename_{implementation_type_1}']}/{row[f'test_filename_{implementation_type_2}']} | return_value_{implementation_type_1}: {row[f'return_value_{implementation_type_1}']} | return_value_{implementation_type_2}: {row[f'return_value_{implementation_type_2}']} | expected: {row[f'expected_value_{implementation_type_1}']}")
    
    diff_df = pd.concat([diff_df, diff_fail])
    
    print('----SUMMARY----')
    print(f"total generated tests {implementation_type_1}: {len(df1)}")
    print(f"total valid tests {implementation_type_1}: {len(df1[df1['test_status'] == 'PASS']) + len(df1[df1['test_status'] == 'FAIL']) }")
    print(f"% valid tests {implementation_type_1}: {(len(df1[df1['test_status'] == 'PASS']) + len(df1[df1['test_status'] == 'FAIL']))/len(df1)}")

    print(f"total generated tests {implementation_type_2}: {len(df2)}")
    print(f"total valid tests {implementation_type_2}: {len(df2[df2['test_status'] == 'PASS']) + len(df2[df2['test_status'] == 'FAIL']) }")
    print(f"% valid tests {implementation_type_2}: {(len(df2[df2['test_status'] == 'PASS']) + len(df2[df2['test_status'] == 'FAIL']))/len(df2)}")

    print(f'diff in test status and mismatch fails in {implementation_type_2} vs {implementation_type_1}', len(diff_df))
    print(f'num pass/fail diffs {implementation_type_2} vs {implementation_type_1}', len(diff_df1) + len(diff_df2))
    print(f'num fail/fail diffs {implementation_type_1} vs {implementation_type_2}', len(diff_fail))

    pass_fail = len(diff_df1) + len(diff_df2)
    fail_fail = len(diff_fail)

    diff_results = pd.concat([diff_df1, diff_df2])
    diff_results = pd.concat([diff_results, diff_fail])

    # update results df   
    for i, row in diff_results.iterrows():
        result = {} 
        result['index'] = row[f'index_{implementation_type_1}']
        result['baseline'] = baseline
        result['diff'] = f'{implementation_type_1} vs {implementation_type_2}'
        result['test_file_path'] = f'../../data/all_generated_tests/{baseline}/processed'
        result['filename'] = row[f'test_filename_{implementation_type_1}']
        result['file_content'] = load_file(f"../../data/all_generated_tests/{baseline}/processed/{row[f'test_filename_{implementation_type_1}']}")
        result[f'test_status_{implementation_type_1}'] = row[f'test_status_{implementation_type_1}']
        result[f'test_status_{implementation_type_2}'] = row[f'test_status_{implementation_type_2}']
        result[f'test_outcome_{implementation_type_1}'] = row[f'test_outcome_{implementation_type_1}']
        result[f'test_outcome_{implementation_type_2}'] = row[f'test_outcome_{implementation_type_2}']
        result['expected_value'] = row[f'expected_value_{implementation_type_1}']
        result[f'return_val_{implementation_type_1}'] = row[f'return_value_{implementation_type_1}']
        result[f'return_val_{implementation_type_2}'] = row[f'return_value_{implementation_type_2}']
        
        result = pd.DataFrame(result, index = [0])
        results_df = pd.concat([results_df, result], ignore_index=True)

    results_df = results_df.reset_index()
    diff_df.to_json(f'{path}/diff_{baseline}_{implementation_type_1}_{implementation_type_2}.json', orient='records')
    #diff_df.to_csv(f'{path}/diff_{baseline}.csv')

    return results_df, pass_fail, fail_fail

def combine_results(path):
    df1 = pd.read_csv(f'{path}/diff_tests_results_linux_ubpf.csv')
    df2 = pd.read_csv(f'{path}/diff_tests_results_ubpf_windows.csv')
    df3 = pd.read_csv(f'{path}/diff_tests_results_linux_windows.csv')

    res_df = pd.concat([df1, df2])
    res_df = pd.concat([res_df, df3])
    res_df.to_csv(f'{path}/diff_tests_results.csv')


if __name__ == "__main__":
    
    path = './output_logs'
    #combine_results(path)

    implementation_type_1 = 'ubpf'
    implementation_type_2 = 'windows'

    baselines = ['3shot_random', 'target_section', 'random_instruction_permutation_3', 'random_instruction_permutation_5', 'random_instruction_permutation_10',  'prompt_chain', 'prompt_chain_instruct', 'bug_centric', 'code_description', 'code_description_diff', 'code_diff', 'bug_guided_code_diff']

    columns=['baseline', 'diff', 'test_file_path', 'filename', 'file_content', 'test_status_windows',  'test_status_linux', 'test_status_ubpf', 'test_outcome_windows',  'test_outcome_linux', 'test_outcome_ubpf', 'expected_value', 'return_val_windows', 'return_val_linux', 'return_val_ubpf', 'notes']
    
    diff_tests_results = pd.DataFrame(columns=columns)
    
    results_df = pd.DataFrame(columns=columns)
    total_pass_fail = 0
    total_fail_fail = 0

    for baseline in baselines:
        results, pass_fail, fail_fail = main(baseline, implementation_type_1, implementation_type_2, results_df, path)
        diff_tests_results = pd.concat([diff_tests_results, results])
    
        total_pass_fail += pass_fail
        total_fail_fail += fail_fail
    
    print('----------final---------')
    print('final diff results len', len(diff_tests_results))
    print('total_pass_fail', total_pass_fail)
    print('total_fail_fail', total_fail_fail)

    diff_tests_results.to_csv(f'{path}/diff_tests_results_{implementation_type_1}_{implementation_type_2}.csv')
    

# to run 
# python3 differential_testing.py | tee output_differential_tests_linux_ubpf.txt
# python3 differential_testing.py | tee output_differential_tests_ubpf_windows.txt
# python3 differential_testing.py | tee output_differential_tests_linux_windows.txt

