import os
import subprocess
import pandas as pd


def main():
    
    root_path = 'C:/ebpf/data/all_generated_tests/'
    output_path = 'C:/ebpf/code/evaluation/output_logs/'
    
    
    baseline_type = ['3shot_random', 'target_section', 'random_instruction_permutation_3', 'random_instruction_permutation_5', 'random_instruction_permutation_10',  'prompt_chain', 'prompt_chain_instruct', 'bug_centric', 'code_description', 'code_description_diff', 'code_diff', 'bug_guided_code_diff'][-1]

    generated_test_files = [file for file in os.listdir(f'{root_path}/{baseline_type}/processed')]
    print(generated_test_files)

    results = pd.DataFrame(columns=['index', 'test_file_path', 'test_filename', 'output', 'error_message'], index=range(len(generated_test_files)))

    for i, file in enumerate(generated_test_files):
        try:
            print()
            print(f'Running test {i} of {len(generated_test_files)}: {file}')

            cmd = f'sudo build/bin/bpf_conformance_runner --test_file_path {root_path}/{baseline_type}/processed/{file} --plugin_path build/bin/libbpf_plugin'
            script_results = subprocess.run(cmd, shell=True, capture_output=True, text=True, timeout=60)

            print('OUTPUT:')
            print(script_results.stdout) 

            print('ERROR:')
            print(script_results.stderr) 
            
            results.loc[i, 'index'] = i
            results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
            results.loc[i, 'test_filename'] = file
            results.loc[i, 'output'] = script_results.stdout.strip()
            results.loc[i, 'error_message'] = script_results.stderr.strip()

        except Exception as e:
            print(f'Unexpected error. Exception: {e}')

        results.to_json(f'{output_path}/output_exec_tests_{baseline_type}_linux.json', orient='records')            


if __name__ == "__main__":
    main()

# Copy file to bpf_conformance folder

# To run
# python3 -u run_tests_linux.py 2>&1 | tee ./output_exec_tests_bug_guided_code_diff_linux.txt

