import os
import subprocess
import pandas as pd


def main():

    root_path = 'C:/ebpf/data/all_generated_tests/'
    output_path = 'C:/ebpf/code/evaluation/output_logs/'


    baseline_type = ['3shot_random', 'target_section', 'random_instruction_permutation_3', 'random_instruction_permutation_5', 'random_instruction_permutation_10',  'prompt_chain', 'prompt_chain_instruct', 'bug_centric', 'code_description', 'code_description_diff', 'code_diff', 'bug_guided_code_diff'][-1]


    generated_test_files = [file for file in os.listdir(f'{root_path}/{baseline_type}/processed')]
    print(generated_test_files)

    results = pd.DataFrame(columns=['index', 'test_file_path', 'test_filename', 'output', 'error_message'], index=range(len(generated_test_files)))

    start = 0
    for i, file in enumerate(generated_test_files[start:]):
        try:
            print()
            print(f'Running test {start+ i} of {len(generated_test_files)}: {file}')

            cmd = f'.\\bpf_conformance_runner.exe --test_file_path {root_path}/{baseline_type}/processed/{file} --cpu_version v4 --exclude_regex local --plugin_path bpf2c_plugin.exe --debug true --plugin_options "--include C:/ebpf-for-windows/include"'
            
            
            # the following tests cause the plugin to hang and are therefore labeled as potential crashes 
            if baseline_type == 'target_section' and file in ['EXIT_9.data']:#, 'JA_5.data', 'JA_6.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue
            
            if baseline_type == '3shot_random' and file in ['JA_5.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue
            
            if baseline_type == 'random_instruction_permutation_3' and file in ['OR_LSH_JA_2.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue
            
            if baseline_type == 'random_instruction_sequence_3' and file in ['RSH_DIV_JSGT_8.data', 'RSH_DIV_JSGT_9.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue
            
            if baseline_type == 'random_instruction_permutation_5' and file in ['EXIT_SDIV_OR_AND_LSH_0.data', 'EXIT_SDIV_OR_AND_LSH_2.data', 'EXIT_SDIV_OR_AND_LSH_4.data', 'EXIT_SDIV_OR_AND_LSH_7.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            if baseline_type == 'prompt_chain' and file in ['JA_8.data', 'JGT_9.data', 'JLE_11.data', 'JLT_8.data', 'JNE_4.data', 'JSET_4.data', 'JSGE_2.data', 'JSLE_8.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            if baseline_type == 'prompt_chain_instruct' and file in ['EXIT_13.data', 'JA_7.data', 'JEQ_14.data', 'JSGT_8.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            if baseline_type == 'bug_centric' and file in [ 'JA_TailCallBugs_0.data','JA_EndiannessConversionBugs_9.data','JA_IncorrectInstructionEncoding_1.data', 'JA_ArithmeticOperationBugs_9.data', 'JA_IncorrectInstructionMapping_1.data', 'JA_ShiftOperationBugs_7.data', 'JLT_ErrorCheckingOmission_0.data', 'JLT_ErrorCheckingOmission_7.data', 'JLT_StackLayoutErrors_8.data', 'JNE_IncorrectAssumptionsaboutHardwareorArchitecture_4.data', 'JNE_RegisterHandlingBugs_14.data', 'JSGE_IncorrectInstructionMapping_4.data', 'MUL_TailCallBugs_2.data', 'SUB_MemoryAccessErrors_9.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            if baseline_type == 'code_description_' and file in ['JA_2.data', 'JNE_16.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue
            
            if baseline_type == 'code_description_alldiffs_' and file in ['JA_17.data', 'JEQ_8.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            if baseline_type == 'code_description_diff_' and file in ['JA_1.data', 'JA_11.data', 'JA_49.data', 'JSGE_71.data', 'JSGT_8.data', 'JSLE_43.data', 'JSLE_6.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue
            
            if baseline_type == 'code_diff_' and file in ['EXIT_38.data', 'EXIT_78.data', 'JA_31.data', 'JA_32.data', 'JA_33.data', 'JA_35.data', 'JA_37.data', 'JEQ_59.data', 'JSET_28.data', 'JSET_29.data', 'JSET_51.data', 'JSLE_3.data', 'JSLT_54.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            if baseline_type == 'bug_guided_code_diff_' and file in ['AND_RegisterHandlingBugs_18.data']:
                results.loc[i, 'index'] = i
                results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
                results.loc[i, 'test_filename'] = file
                results.loc[i, 'output'] = 'potential crash'
                results.loc[i, 'error_message'] = 'potential crash'
                continue

            script_results = subprocess.run(cmd, capture_output=True, text=True, timeout=60)
            
            print('OUTPUT:')
            print(script_results.stdout) 

            print('ERROR:')
            print(script_results.stderr) 
            
            results.loc[i, 'index'] = i
            results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
            results.loc[i, 'test_filename'] = file
            results.loc[i, 'output'] = script_results.stdout.strip()
            results.loc[i, 'error_message'] = script_results.stderr.strip()

            if (start + i) % 100 == 0:
                results.to_json(f'{output_path}/output_exec_tests_{baseline_type}_windows_{start + i}.json', orient='records')

        except Exception as e:
            print(f'Unexpected error. Exception: {e}')
            results.loc[i, 'index'] = i
            results.loc[i, 'test_file_path'] = f'{root_path}/{baseline_type}/processed'
            results.loc[i, 'test_filename'] = file
            results.loc[i, 'output'] = 'crash'
            results.loc[i, 'error_message'] = str(e)

        results.to_json(f'{output_path}/output_exec_tests_{baseline_type}_windows.json', orient='records')


if __name__ == "__main__":
    main()


# set env variable to run python and restart powershell terminal
# [Environment]::SetEnvironmentVariable("Path", "$env:Path;C:\Program Files\Python312")

# to run file

# python -u run_tests_windows.py 2>&1 | Tee-Object -FilePath ".\output_exec_tests_bug_guided_code_diff_windows.txt"