import os
import sys
sys.path.append('../')
from utils.data_utils import load_as_df, load_json
from utils.model_utils import prompt_model
from collections import Counter



def get_groundtruth_tests(path):
    file_names = sorted([file.split('.')[0] for file in os.listdir(path)])
    return file_names


def get_instructions(path):
    df = load_as_df(path)
    return list(df['instruction'])


def verify_mapping(path, instructions_list, true_test_filenames):
    df = load_as_df(path)
    
    instructions_map = list(df['instruction'])
    instructions_intersection = set.intersection(set(instructions_list), set(instructions_map))
    print(f'Verify Instructions:')
    print(f'Instructions list {len(instructions_list)}: {instructions_list}')
    print(f'Instructions from mapping {len(instructions_map)}: {instructions_map}')
    print(f'Intersection {len(instructions_intersection)}: {instructions_intersection}')

    tests_map = list(df['tests'].apply(lambda x: x.split(',')))
    tests_map = [x.strip() for xs in tests_map for x in xs]
    
    tests_intersection = set.intersection(set(true_test_filenames), set(tests_map))
    extra_true_tests = set(true_test_filenames).difference(tests_intersection)
    extra_tests_map = set(tests_map).difference(tests_intersection)
    
    print(f'Verify tests:')
    print(f'True test list {len(true_test_filenames)}: {true_test_filenames}')
    print(f'Tests from mapping {len(tests_map)}: {tests_map}')
    print(f'Intersection {len(tests_intersection)}: {tests_intersection}')
    print(f'True test - Intersection {len(extra_true_tests)}: {extra_true_tests}')
    print(f'Tests from mapping - Intersection {len(extra_tests_map)}: {extra_tests_map}')
    print(f'Distribution of tests from mapping: {Counter(tests_map)}')
    print(f'Number of instructions with no tests ({len(df[df["tests"] == ""])}): {list(df[df["tests"] == ""]["instruction"])}')
    print(f'Number of tests with no instructions ({len(df[df["instruction"] == "UNKNOWN"]["tests"].values[0].split(","))}): {df[df["instruction"] == "UNKNOWN"]["tests"].values[0].split(",")}')


def main():

    # load variables
    manual = load_json('../context/BPFInstructionSetArchitecture.json')
    print(f'Manual:\n{manual}')

    true_tests_path = '../bpf_conformance_tests'
    true_test_filenames = get_groundtruth_tests(true_tests_path)
    print(f'List of ground truth test file names ({len(true_test_filenames)}): {true_test_filenames}')
    
    instructions_list = get_instructions('../context/instructions_ground_truth.txt')
    print(f'List of instructions  ({len(instructions_list)}): {instructions_list}')
    

    # load model
    model = "gpt-4o"
       

    prompt = f"You are given a list of instructions, along with a list of tests. Create a mapping between the list of instructions and their corresponding and output the results as a table with the following format: Index | Instruction | Tests. List all tests that you did not find a mapping for under UNKNOWN instruction. Make sure you are thorough and you don't leave out any instructions or tests. Here is an example: \n1 | ADD | add, add64, alu-arith, alu64-arith. Here are the list of instructions: {instructions_list}.\n Here are the list of tests: {true_test_filenames}"


    # prompt model
    response = prompt_model(model, prompt)
    
    # print model response
    print(f'-----------------<output>---------------------')
    print(response)



if __name__ == "__main__":
    main()

# to run 
# python -u main_map_instructions_list_to_testfile_name.py 2>&1 | tee output_instruction_tests_map.txt
