import os
import sys
import time
import pandas as pd
sys.path.append('../')
from utils.data_utils import load_pdf, load_file, save_file, load_as_df
from utils.model_utils import get_output_counts_per_instruction
from utils.prompt_utils import extract_manual_instruction_constraints, extract_relevant_code, get_nl_description_of_code, generate_test_descriptions_from_code_descriptions, generate_more_test_descriptions_from_code_descriptions,  generate_test_body

 

def main():
    start_timer = time.time()

    # load variables

    instructions_df = load_as_df('../context/instructions_ground_truth.txt')
    k = 10
    
    # load model
    model = "gpt-4o"
    manual = load_pdf('../context/BPFInstructionSetArchitecture.pdf')
    example_tests = load_file('../examples/random_3shot.txt')

    codefile1 = load_file('../context/code/bpf_jit_compiler_arm_32bit.c') 
    codefile2 = load_file('../context/code/bpf_jit_compiler_x86_32bit.c') 
    
    architecture1 = 'arm32'
    architecture2 = 'x86'

    output_path = f'../../data/all_generated_tests/code_description'

    print('----------------<output>-------------------')
    
    # random 3-shot examples for all instructions 
    for i, row in instructions_df.iterrows():
        start = time.time()
        instruction = row['instruction']
        print(f'instruction {i}: {instruction}')
        instruction_constraints = extract_manual_instruction_constraints(model, manual, instruction)
        print('instruction_constraints')
        print(instruction_constraints)
        print('\n\n')

        # extract code for inst
        src_code1 = extract_relevant_code(model, instruction, instruction_constraints, codefile1, architecture1)
        src_code2 = extract_relevant_code(model, instruction, instruction_constraints, codefile2, architecture2)
        
        # generate nl desc for extracted code
        code1_description = get_nl_description_of_code(model, instruction, instruction_constraints, src_code1, architecture1)
        code2_description = get_nl_description_of_code(model, instruction, instruction_constraints, src_code2, architecture2)
        
        print(f'code and description for {architecture1}')
        print(src_code1)
        print(code1_description)
        print('\n\n')

        print(f'code and description for {architecture2}')
        print(src_code2)
        print(code2_description)
        print('\n\n')

        nl_test_descriptions = generate_test_descriptions_from_code_descriptions(model, instruction, instruction_constraints, code1_description, code2_description, architecture1, architecture2)

        print('nl_test_descriptions')
        print(nl_test_descriptions)
        print('\n\n')
        nl_test_descriptions_list = nl_test_descriptions.split('\n\n')
    
        more_nl_test_descriptions = generate_more_test_descriptions_from_code_descriptions(model, nl_test_descriptions, instruction, instruction_constraints, code1_description, code2_description, architecture1, architecture2)

        print('more nl_test_descriptions')
        print(more_nl_test_descriptions)
        print('\n\n')

        nl_test_descriptions_list.extend(more_nl_test_descriptions.split('\n\n'))
    
        for j, nl_test_description in enumerate(nl_test_descriptions_list):
            # remove newlines from test description
            nl_test_description = ' '.join(nl_test_description.split('\n')).strip()
            print('NL description of test')
            print(nl_test_description)
            
            generated_test = generate_test_body(model, instruction, instruction_constraints, example_tests, nl_test_description)
            # invalid test format, regenerate test
            if '```' in generated_test:
                generated_test = generate_test_body(model, instruction, instruction_constraints, example_tests, nl_test_description)
            # if multiple tests only keep first one
            if '<SEP>' in generated_test:
                generated_test.split('SEP')[0]
                
            # postprocess, move comments in same line as code to a new line
            # having comments in the same line as code (mov32 %r0, -2147483648  # signed value -2147483648) leads to "Invalid number of operands for mnemonic" error
            generated_test = generated_test.replace('#', '\n#')
            # print model response
            print('Generated test')
            print(generated_test)
            print('\n\n')
            generated_test = f"# {nl_test_description}\n{generated_test}"

            # save output in file
            save_file(generated_test, f"{output_path}/processed/{instruction}_{j}.data")

        end = time.time()

        # log progress 
        print(f'{i}: {instruction} | generated_tests count: {len(nl_test_descriptions_list)} | generated tests path: {output_path} | time taken: {end - start}')

    end_timer = time.time()
    print(f'total time: {end_timer - start_timer}')
    get_output_counts_per_instruction(f'{output_path}/processed')


if __name__ == "__main__":
    main()


# to run file
# python -u main_code_description.py 2>&1 | tee output_code_description.txt
