import os
import sys
import time
sys.path.append('../')
from utils.data_utils import load_file, load_as_df, save_file, load_pdf, format_and_save_outputs
from utils.model_utils import prompt_model, is_valid_model_output, get_output_counts_per_instruction



# prompt model 
def prompt_model_single_instruction(query_instruction, manual, examples, model):    
    prompt = f"Given a document that specifies the BPF instruction set architecture (ISA), your task is to generate tests for each of the instructions. Here is the manual:\n{manual}. Here are some example tests:\n{examples}. Generate 10 tests for the {query_instruction} instruction using the same format, include a one line description of what the test does and make sure you handle all edge cases. Use <SEP> tag to separate the tests."
    
    # prompt model
    response = prompt_model(model, prompt)
    
    return response


def main():
    start_timer = time.time()

    # load variables
    manual = load_pdf('../context/BPFInstructionSetArchitecture.pdf')
    examples = load_file('../examples/random_3shot.txt')
    instructions_df = load_as_df('../context/instructions_ground_truth.txt')

    
    print(instructions_df)
    k = 10
    output_path = f'../../data/all_generated_tests/3shot_random'

    # load model
    model = "gpt-4o"


    print('----------------<output>-------------------')
    
    existing_files = [file for file in os.listdir(f'{output_path}/raw_output/')]
    
    
    # random 3-shot examples for all instructions 
    for i, row in instructions_df.iterrows():
        #print(i, row)
        start = time.time()
        query_instruction = row['instruction']
        
        if any(query_instruction in file for file in existing_files):
            print(f'{i}: {query_instruction} already processed | generated tests path: {output_path}/raw_output/{query_instruction}_10.txt')
            continue

        # if model output format is not valid, retry
        while True:
            # random 3-shot examples for query instruction
            output = prompt_model_single_instruction(query_instruction, manual, examples, model)

            if is_valid_model_output(output, k):
                break
            else:
                print('Invalid model output, prompting llm again')

        # format output
        format_and_save_outputs(output, f'{output_path}/processed', query_instruction, k)
        end = time.time()

        # log progress 
        print(f'{i}: {query_instruction} | generated tests path: {output_path} | time taken: {end - start}')


    end_timer = time.time()
    
    print(f'total time: {end_timer - start_timer}')
    
    get_output_counts_per_instruction(f'{output_path}/processed')


if __name__ == "__main__":
    main()


# to run file
# python -u main_entire_isa_3shot_random.py 2>&1 | tee output_entire_isa_3shot_random.txt
