import os
import sys
import time
sys.path.append('../')
from utils.data_utils import load_pdf, load_file, save_file, load_as_df
from utils.model_utils import prompt_model, get_output_counts_per_instruction


def main():
    start_timer = time.time()

    # load variables
    instructions_df = load_as_df('../context/instructions_ground_truth.txt')
    print(instructions_df)
    k = 10
    output_path = f'../../data/all_generated_tests/prompt_chain_instruct'

    # load model
    model = "gpt-4o"
    manual = load_pdf('../context/BPFInstructionSetArchitecture.pdf')
    examples = load_file('../examples/random_3shot.txt')



    print('----------------<output>-------------------')
    
    # random 3-shot examples for all instructions 
    for i, row in instructions_df.iterrows():
        #print(i, row)
        start = time.time()
        instruction = row['instruction']

        prompt_doc_summary = f"You are an expert in ebpf. Here is a ISA document that details the exact specifications that need to be followed for eBPF.\n\n{manual}\n\nThis document is used as a reference to implement the ebpf framework for windows and linux. The goal is to find differential tests that returns different outputs in the linux and windows implementation. Can you extract the key points relevant to the {instruction} instruction from the documentation. Make sure you are extremely specific, and extract all the constraints and conditions that strictly need to be followed when implementing the {instruction} instruction. The goal is to use these constraints to generate differential tests, so make sure you include details that can be easily missed or misinterpreted so we can test for all the edge cases."
        
        # prompt model
        output_doc_summary = prompt_model(model, prompt_doc_summary)
        print('doc summary')
        print(output_doc_summary)
        print('\n\n')

        prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in the linux and windows implementation of ebpf. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{output_doc_summary}\n\nYour goal is to use these constraints to generate differential tests so we can test for all the edge cases that can lead to different outputs in linux and in windows. Can you generate a natural language description of tests that can result in differential behaviour in linux and windows implementation. Make sure to also include information on the edge case or specific constraint that the test is checking for. Only provide the list of tests as output."

        output_test_descriptions = prompt_model(model, prompt_test_descriptions)
        print('some tests')
        print(output_test_descriptions)
        print('\n\n')
        gen_nl_tests = output_test_descriptions.split('\n\n')
        #print(gen_nl_tests)

        # here are some tests, generate other tests
        prompt_more_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in the linux and windows implementation of ebpf. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{output_doc_summary}\n\nYour goal is to use these constraints to generate differential tests so we can test for all the edge cases that can lead to different outputs in linux and in windows. Here are some tests that currently exist.\n\n{output_test_descriptions}\n\nCan you generate other tests that are currently not included which can result in differential behaviour in linux and windows implementation. Make sure to also include information on the edge case or specific constraint that the test is checking for. Only provide the list of tests as output."

        output_more_test_descriptions = prompt_model(model, prompt_more_test_descriptions)
        print('more tests')
        print(output_more_test_descriptions)
        print('\n\n')

        gen_nl_tests.extend(output_more_test_descriptions.split('\n\n'))

        for j, gen_nl_test in enumerate(gen_nl_tests):
            # remove newlines from test description
            gen_nl_test = ' '.join(gen_nl_test.split('\n')).strip()
            print('NL description of test')
            print(gen_nl_test)
            prompt_gen_test = f"""You are an expert in ebpf. The goal is to find differential tests that returns different outputs in the linux and windows implementation of ebpf. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{output_doc_summary}\n\nYour goal is to use these constraints to generate differential tests so we can test for all the edge cases that can lead to different outputs in linux and in windows. Here are some example of existing tests for the {instruction} instruction.\n\n{examples}\n\nHere is a description of a test that can result in differential behaviour in linux and windows implementation for the {instruction} instruction.\n{gen_nl_test}\n\n
            Generate the code for the test in the same format as the example. You are required to strictly follow these instructions when generating the test.\n1. Make sure you only generate one test.\n2. Do not include comments in the same line as code.\n3. Instructions default to 64 bit operations, for example, use mov instead of mov64. mov64 is not recognised.\n4. Include valid return values in the result section of the code, do not leave comments or text there.\n5. Make sure you use valid registers.\n6. Make sure you use valid labels when using jump instructions.\n7. Make sure the final result is in the register r0.\n\nOnly generate the test code so it can be directly executed, and comment out any natural language descriptions describing what the test does."""
            
            output_gen_test = prompt_model(model, prompt_gen_test)
            # invalid test format, regenerate test
            if '```' in output_gen_test:
                output_gen_test = prompt_model(model, prompt_gen_test)
            # if multiple tests only keep first one
            if '<SEP>' in output_gen_test:
                output_gen_test.split('SEP')[0]
            
            # postprocess, move comments in same line as code to a new line
            # having comments in the same line as code (mov32 %r0, -2147483648  # signed value -2147483648) leads to "Invalid number of operands for mnemonic" error
            output_gen_test = output_gen_test.replace('#', '\n#')
            # print model response
            print('Generated test')
            print(output_gen_test)
            print('\n\n')
            output_gen_test = f"# {gen_nl_test}\n{output_gen_test}"
            # save output in file
            save_file(output_gen_test, f'{output_path}/processed/{instruction}_{j}.data')


        end = time.time()

        # log progress 
        print(f'{i}: {instruction} | generated_tests count: {len(gen_nl_tests)} | generated tests path: {output_path} | time taken: {end - start}')

        

    end_timer = time.time()
    
    print(f'total time: {end_timer - start_timer}')

    get_output_counts_per_instruction(f'{output_path}/processed')


if __name__ == "__main__":
    main()


# to run file
# python -u main_prompt_chain_instruct.py 2>&1 | tee output_prompt_chain_instruct_2.txt
