import sys
import time
sys.path.append('../')
from utils.data_utils import load_as_df, load_json, fetch_relevant_documentation, get_examples_context, format_and_save_outputs
from utils.model_utils import prompt_model, is_valid_model_output, get_output_counts_per_instruction



# prompt model 
def prompt_model_single_instruction(query_instruction, manual, examples, model):    
    prompt = f"Given a description of the instruction class extracted from the BPF instruction set architecture (ISA), your task is to generate tests for the {query_instruction} instruction. Here is the relevant documentation:\n{manual}. Here are some example tests for this instruction:\n{examples}. Generate 10 more tests for the {query_instruction} instruction using the same format, include a one line description of what the test does and make sure you handle all edge cases. Use <SEP> tag to separate the tests."
    
    # prompt model
    response = prompt_model(model, prompt)
    
    return response


def main():
    start_timer = time.time()

    # load variables
    manual = load_json('../context/BPFInstructionSetArchitecture.json')
    instructions_df = load_as_df('../context/instructions_ground_truth.txt')
    print(instructions_df)
    mapping_df = load_as_df('../context/nl_test_lists_mapping_with_gt_instructions.txt')
    print(mapping_df)
    true_tests_path = '../bpf_conformance_tests'
    k = 10

    output_path = f'../../data/all_generated_tests/target_section'

    # load model
    model = "gpt-4o"

    print('----------------<output>-------------------')
    
    # random 3-shot examples for all instructions 
    for i, row in instructions_df.iterrows():
        #print(i, row)
        start = time.time()
        query_instruction = row['instruction']

        reference_section = instructions_df[instructions_df['instruction'] == query_instruction]['reference'].values[0]
        
        relevant_documentation = fetch_relevant_documentation(manual, reference_section)
    
        relevant_tests = mapping_df[mapping_df['Instruction'] == query_instruction]['Tests'].values[0].split(',')
    
        examples = get_examples_context(relevant_tests, true_tests_path)

        # if model output format is not valid, retry
        while True:
            output = prompt_model_single_instruction(query_instruction, relevant_documentation, examples, model)
            if is_valid_model_output(output, k):
                break
            else:
                print('Invalid model output, prompting llm again')

        # format output
        format_and_save_outputs(output, f'{output_path}/processed', query_instruction, k)

        end = time.time()

        # log progress 
        print(f'{i}: {query_instruction} | generated tests path: {output_path} | time taken: {end - start}')


    end_timer = time.time()
    
    print(f'total time: {end_timer - start_timer}')

    get_output_counts_per_instruction(f'{output_path}/processed')

if __name__ == "__main__":
    main()


# to run file
# python -u main_target_section_isa_specific_tests.py 2>&1 | tee output_target_section_isa_specific_tests.txt
