from langchain_community.document_loaders import SeleniumURLLoader
from langchain_community.document_loaders import PyMuPDFLoader
import pandas as pd
import json
import pathlib

# load file given path
def load_file(file_path):
    with open(file_path, 'r') as f:
        data = f.read()
    return data


# load data from url
def load_from_url(url):
    url = [url] # UnstructuredURLLoader expects list of urls
    loader = SeleniumURLLoader(urls=url)
    data = loader.load()
    return data


# load manual from pdf
def load_pdf(file_path):
    loader = PyMuPDFLoader(file_path)
    data = loader.load()
    return data


# load manual from json
def load_json(file_path):
    with open(file_path, 'r') as f:
        data = json.loads(json.load(f))
    return data


# load gpt4 output table (| separated) as df
def load_as_df(file_path):
    content = load_file(file_path)
    columns = content.split('\n')[0].split('|') # first row is the header
    columns = [column.strip() for column in columns]
    df = pd.DataFrame(columns = columns)
    index = 0
    for row in content.split('\n')[1:]: # skip first row, headers
        for i, value in enumerate(row.split('|')):
            df.loc[index, columns[i]] = value.strip()
        index += 1
    df = df.reset_index()
    df = df[columns]
    return df


# save output to file
def save_file(content, path):
    with open(path, 'w') as output:
        output.write(content)


def find_section_by_subsections(doc, name):
    # Helper function to recursively search subsections and subsubsections
    def search_subsections(subsections, section_name):
        for subsec_name, subsec_value in subsections.items():
            if subsec_name == name:
                return section_name
            # Check if there are subsubsections
            if 'subsubsections' in subsec_value:
                for subsubsec_name in subsec_value['subsubsections'].keys():
                    if subsubsec_name == name:
                        return section_name
        return None    

    # if name is a section, return section
    if name in doc.keys():
        return name
    
    # Iterate over the sections
    for section_name, section_value in doc.items():
        # Check if the subsection exists at the first level
        if 'subsections' in section_value:
            result = search_subsections(section_value['subsections'], section_name)
            if result:
                return result

    return None


def fetch_relevant_documentation(manual, subsection_name):
    subsection_name = " ".join(subsection_name.split(' ')[1: ]) # remove section number
    section_name = find_section_by_subsections(manual, subsection_name).strip()
    #print(f'relevant section extracted: {section_name}')
    context = ''
    context += manual[section_name]['metadata']['title']
    context += manual[section_name]['page_content']
    for name, subsection in manual[section_name]['subsections'].items():
        context += subsection['metadata']['title']
        context += subsection['page_content']
        for name, subsubsection in subsection['subsubsections'].items():
            context += subsubsection['metadata']['title']
            context += subsubsection['page_content']
    
    return context


def format_and_save_outputs(output, path, instruction, k):
    for i, test in enumerate(output.split('<SEP>')):
        if i < k:
            if test.strip() != '':
                save_file(test.strip(), f'{path}/{instruction}_{i}.data')
        else:
            return


def create_output_dir_path(path):
    print(f'creating output directory path: {path}')
    pathlib.Path(path).mkdir(parents=True, exist_ok=True)


def get_examples_context(test_names, path):
    examples_context = ''
    for test_name in test_names:
        test = load_file(f'{path}/{test_name.strip()}.data')
        examples_context += f'# test: {test_name.strip()}.data\n'
        examples_context += test + '\n\n'
    return examples_context
