from tenacity import retry, wait_random_exponential, stop_after_attempt, before_sleep_log
import logging
import os 
from collections import Counter
from openai import OpenAI


logger = logging.getLogger()
logging.basicConfig(level=logging.INFO)

# Retry: Randomly wait up to 2^x * 1 seconds between each retry until the range reaches 60 seconds, then randomly up to 60 seconds afterwards 
# Stop after 2000 attempts
@retry(reraise=True, before_sleep=before_sleep_log(logger, logging.INFO),wait=wait_random_exponential(multiplier=1, max=60), stop=stop_after_attempt(2000))
def prompt_model(model, prompt):
    
    # open ai public endpoint
    client = OpenAI()
    completion = client.chat.completions.create(
        model=model,
        messages=[
            {
                "role": "user",
                "content": prompt
            }
        ]  
    )

    return completion.choices[0].message.content
  

def is_valid_model_output(output, k):
    if '```' in output:
        return False
    generations = output.split('<SEP>')
    generations = [generation for generation in generations if generation.strip() != '']
    if len(generations) != k:
        print(f"number of generations not equal to expected value {len(generations)} != {k}")
        return False
    return True


def get_output_counts_per_instruction(path):
    existing_files = [file for file in os.listdir(path)]
    print(existing_files)
    instructions = [f for file in existing_files for f in file.split('_')[:-1]]
    print(Counter(instructions))
