import sys
sys.path.append('../')
from utils.model_utils import prompt_model


def extract_manual_instruction_constraints(model, manual, instruction):
    prompt_doc_summary = f"You are an expert in ebpf. Here is a ISA document that details the exact specifications that need to be followed for eBPF.\n\n{manual}\n\nThis document is used as a reference to implement the ebpf framework for different architectures. The goal is to find differential tests that returns different outputs in different implementations. Can you extract the key points relevant to the {instruction} instruction from the documentation. Make sure you are extremely specific, and extract all the constraints and conditions that strictly need to be followed when implementing the {instruction} instruction. The goal is to use these constraints to generate differential tests, so make sure you include details that can be easily missed or misinterpreted so we can test for all misinterpretations and nuances."

    # prompt model
    instruction_constraints = prompt_model(model, prompt_doc_summary)
    return instruction_constraints


def generate_test_descriptions(model, instruction, instruction_constraints, code=False):
    if not code:
        prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYour goal is to generate differential tests so we can test for nuances in the two implementations based on the natural language specifications. Can you generate a natural language description of tests that can result in differential behaviour in two implementations. Make sure to also include information on the specific constraint that the test is checking for. Only provide the list of tests as output."
    else:
        pass
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions


def generate_more_test_descriptions(model, instruction, instruction_constraints, nl_test_descriptions):
    # here are some tests, generate other tests
    prompt_more_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYour goal is to generate differential tests so we can test for nuances in the two implementations based on the natural language specifications. Here are some tests that currently exist.\n\n{nl_test_descriptions}\n\nCan you generate other tests that are currently not included which can result in differential behaviour in the two implementations. Make sure to also include information on the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    more_nl_test_descriptions = prompt_model(model, prompt_more_test_descriptions)
    return more_nl_test_descriptions


def generate_test_descriptions_from_bug(model, instruction, instruction_constraints, bug_class, bug_description):
    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the two implementations based on the natural language specifications. Focus on {bug_class} when generating the tests. {bug_description}. Can you generate a natural language description of tests that can result in differential behaviour in the two implementations for {bug_class}. Make sure to also include information on how the test relates to the {bug_class} and the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions


def generate_test_descriptions_from_code(model, instruction, instruction_constraints, code):
    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nHere is the source code for {instruction} instruction.\n{code}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the code implementation based on the natural language specifications. Can you generate a natural language description of tests that can result in differential behaviour in the implementation for {instruction} instruction. Make sure to also include information on the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions


def generate_test_descriptions_from_bug_and_code(model, instruction, instruction_constraints, bug_class, bug_description, code):
    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\n\n\nHere is the source code for {instruction} instruction.\n{code}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the code implementation based on the natural language specifications. Focus on {bug_class} when generating the tests. {bug_description}. Can you generate a natural language description of tests that can result in differential behaviour in the two implementations for {bug_class} for {instruction} instruction. Make sure to also include information on how the test relates to the {bug_class} and the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions


def generate_test_descriptions_from_code_descriptions(model, instruction, instruction_constraints, code1_description, code2_description, architecture1, architecture2):
    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a detailed line by line description of the code implementing the {instruction} instruction in two different implementations. Here is the description of the code implemented for {architecture1}: {code1_description}\n\nHere is the description of the code implemented for {architecture2}: {code2_description}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the code implementation based on the natural language specifications extracted from the manual along with the two natural language descriptions of the code implementations for {instruction} instruction in {architecture1} and {architecture2}. Can you generate a natural language description of tests that can result in differential behaviour in the implementation for {instruction} instruction. Make sure to also include information on the lines of code and the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions


def generate_more_test_descriptions_from_code_descriptions(model, nl_test_descriptions, instruction, instruction_constraints, code1_description, code2_description, architecture1, architecture2):
    prompt_more_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a detailed line by line description of the code implementing the {instruction} instruction in two different implementations. Here is the description of the code implemented for {architecture1}: {code1_description}\n\nHere is the description of the code implemented for {architecture2}: {code2_description}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the code implementation based on the natural language specifications extracted from the manual along with the two natural language descriptions of the code implementations for {instruction} instruction in {architecture1} and {architecture2}. Here are some tests that currently exist.\n\n{nl_test_descriptions}\n\nCan you generate a natural language description of other tests that are currently not included which can result in differential behaviour in the implementation for {instruction} instruction. Make sure to also include information on the lines of code and the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    more_nl_test_descriptions = prompt_model(model, prompt_more_test_descriptions)
    return more_nl_test_descriptions


def generate_test_body(model, instruction, instruction_constraints, example_tests, nl_test_description):
    prompt_gen_test = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYour goal is to generate differential tests so we can test for nuances in the two implementations based on the natural language specifications. Here are some examples of existing tests for various instructions.\n\n{example_tests}\n\nHere is a description of a test that can result in differential behaviour in the two implementations for the {instruction} instruction.\n{nl_test_description}\n\nGenerate the code for the test in the same format as the example. You are required to strictly follow these instructions when generating the test.\n1. Make sure you only generate one test.\n2. Do not include comments in the same line as code.\n3. Instructions default to 64 bit operations, for example, use mov instead of mov64. mov64 is not recognised.\n4. Include valid return values in the result section of the code, do not leave comments or text there.\n5. Make sure you use valid registers.\n6. Make sure you use valid labels when using jump instructions.\n7. Make sure the final result is in the register r0.\n\nOnly generate the test code so it can be directly executed, and comment out any natural language descriptions describing what the test does."
            
    generated_test = prompt_model(model, prompt_gen_test)
    return generated_test


def extract_relevant_code(model, instruction, instruction_constraints, codefile, architecture):
    prompt_code_extraction = f"You are an expert in ebpf. Here is the source code of the bpf jit compiler implemented for {architecture} architecture.\n{codefile}\n\nHere are the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nUsing these points as reference, can you extract the relevant code for {instruction} instruction from the code implementation."
    
    # prompt model
    relevant_code = prompt_model(model, prompt_code_extraction)
    return relevant_code


def get_nl_description_of_code(model, instruction, instruction_constraints, code, architecture):
    prompt_describe_code = f"You are an expert in ebpf. Here are the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nHere is the source code relevant to the {instruction} instruction in the bpf jit compiler implemented for {architecture} architecture.\n{code}\n\nUsing the natural language constraints as reference, can you provide a line by line explaination of the code implementing the {instruction} instruction."
    
    # prompt model
    code_description = prompt_model(model, prompt_describe_code)
    return code_description


def generate_differences_from_code_snippets(model, instruction, instruction_constraints, src_code1, src_code2, architecture1, architecture2):
    prompt_code_descriptions_diff = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with the relevant code implementing the {instruction} instruction in two different implementations. Here is the code implementation for {architecture1}: {src_code1}\n\nHere is the code implementation for {architecture2}: {src_code2}\n\nIdentify differences in the two code implementations and return the output as a list of differences between the two implementations."
    
    code_descriptions_diff = prompt_model(model, prompt_code_descriptions_diff)
    return code_descriptions_diff


def generate_test_descriptions_from_code_diff(model, instruction, instruction_constraints, code_diff, architecture1, architecture2):

    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a list of differences in the code implementations for the {instruction} instruction in two different implementations. Here is a key difference in the way {instruction} instruction was implemented in {architecture1} and {architecture2}: {code_diff}\n\nYour goal is to generate 10 unique differential tests so we can test for this difference in the code implementation for {instruction} instruction in {architecture1} and {architecture2}. Can you generate a natural language description of tests that can result in differential behaviour based on the difference in the code implementations for {instruction} instruction. Make sure to reason about the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions

def generate_more_test_descriptions_from_code_diff(model, nl_test_descriptions, instruction, instruction_constraints, code_diff, architecture1, architecture2):
    prompt_more_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a list of differences in the code implementations for the {instruction} instruction in two different implementations. Here is a key difference in the way {instruction} instruction was implemented in {architecture1} and {architecture2}: {code_diff}\n\nYour goal is to generate 10 unique differential tests so we can test for this difference in the code implementation for {instruction} instruction in {architecture1} and {architecture2}. Here are some tests that currently exist.\n\n{nl_test_descriptions}\n\nCan you generate a natural language description of other tests that are currently not included which can result in differential behaviour based on the difference in the code implementations for {instruction} instruction. Make sure to reason about the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    more_nl_test_descriptions = prompt_model(model, prompt_more_test_descriptions)
    return more_nl_test_descriptions


def generate_differences_from_code_descriptions(model, instruction, instruction_constraints, code1_description, code2_description, architecture1, architecture2):
    prompt_code_descriptions_diff = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a detailed line by line description of the code implementing the {instruction} instruction in two different implementations. Here is the description of the code implemented for {architecture1}: {code1_description}\n\nHere is the description of the code implemented for {architecture2}: {code2_description}\n\nIdentify differences in the two implementations based on the natural language descriptions of the code and return the output as a list of differences between the two implementations."
    
    code_descriptions_diff = prompt_model(model, prompt_code_descriptions_diff)
    return code_descriptions_diff


def generate_test_descriptions_from_all_code_descriptions_diff(model, instruction, instruction_constraints, code_descriptions_diff, architecture1, architecture2):

    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a list of differences in the code implementations for the {instruction} instruction in two different implementations. Here is the list of differences for {architecture1} and {architecture2}: {code_descriptions_diff}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the code implementation based on the natural language specifications extracted from the manual along with differences in the code implementations for {instruction} instruction in {architecture1} and {architecture2}. Can you generate a natural language description of tests that can result in differential behaviour in the implementation for {instruction} instruction. Make sure to also include information on the lines of code and the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions

def generate_more_test_descriptions_from_all_code_descriptions_diff(model, nl_test_descriptions, instruction, instruction_constraints, code_descriptions_diff, architecture1, architecture2):
    prompt_more_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a list of differences in the code implementations for the {instruction} instruction in two different implementations. Here is the list of differences for {architecture1} and {architecture2}: {code_descriptions_diff}\n\nYour goal is to generate 10 unique differential tests so we can test for nuances in the code implementation based on the natural language specifications extracted from the manual along with differences in the code implementations for {instruction} instruction in {architecture1} and {architecture2}. Here are some tests that currently exist.\n\n{nl_test_descriptions}\n\nCan you generate a natural language description of other tests that are currently not included which can result in differential behaviour in the implementation for {instruction} instruction. Make sure to also include information on the lines of code and the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    more_nl_test_descriptions = prompt_model(model, prompt_more_test_descriptions)
    return more_nl_test_descriptions

def generate_test_descriptions_from_code_descriptions_diff(model, instruction, instruction_constraints, code_descriptions_diff, architecture1, architecture2):

    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a list of differences in the code implementations for the {instruction} instruction in two different implementations. Here is a key difference in the way {instruction} instruction was implemented in {architecture1} and {architecture2}: {code_descriptions_diff}\n\nYour goal is to generate 10 unique differential tests so we can test for this difference in the code implementation for {instruction} instruction in {architecture1} and {architecture2}. Can you generate a natural language description of tests that can result in differential behaviour based on the difference in the code implementations for {instruction} instruction. Make sure to reason about the specific constraint that the test is checking for. Only provide the list of tests as output."
    
    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions


def generate_test_descriptions_from_bug_code_diff(model, instruction, instruction_constraints, code_diff, bug_class, bug_description, architecture1, architecture2):
    
    prompt_test_descriptions = f"You are an expert in ebpf. The goal is to find differential tests that returns different outputs in different implementations of ebpf, for instance, ebpf for windows vs linux. You are provided with the key points relevant to the {instruction} instruction, along with a set of constraints that must be strictly followed when implementing the {instruction} instruction.\n\n{instruction_constraints}\n\nYou are provided with a list of differences in the code implementations for the {instruction} instruction in two different implementations. Here is a key difference in the way {instruction} instruction was implemented in {architecture1} and {architecture2}: {code_diff}\n\nYour goal is to generate 10 unique differential tests so we can test for this difference in the code implementation for {instruction} instruction in {architecture1} and {architecture2}. Focus on {bug_class} when generating the tests. {bug_description}. Can you generate a natural language description of tests that can result in differential behaviour based on the difference in the code implementations for {instruction} instruction for {bug_class}. Make sure to  reason about the specific constraint that the test is checking for and include information on how the test relates to the {bug_class}. Only provide the list of tests as output."

    nl_test_descriptions = prompt_model(model, prompt_test_descriptions)
    return nl_test_descriptions